/**
 * Copyright (c) 2006 Macacos.org. All Rights Reserved
 *
 * This library is free software; you can redistribute it and/or modify it under the terms of the 
 * GNU Lesser General Public License as published by the Free Software Foundation; either version 
 * 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See 
 * the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this library;
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, 
 * MA  02110-1301  USA
 * 
 * You can find the license also here: http://www.gnu.org/copyleft/lesser.html
 * 
 * 
 * Created on Sep 15, 2006 by Thiago Tonelli Bartolomei
 * -------------------------------------------------
 *           \                                                                           
 *            \                                                                          
 *               __                                                                      
 *          w  c(..)o                                                                    
 *           \__(o)                                                                      
 *               /\                                                                      
 *            w_/(_)-~                                                                   
 *                /|                                                                     
 *               | \                                                                     
 *               m  m   
 */
package healthwatcher.aspects.concurrency;

import healthwatcher.model.complaint.DiseaseType;
import healthwatcher.model.complaint.Symptom;
import healthwatcher.model.employee.Employee;
import healthwatcher.business.employee.*;
import healthwatcher.business.complaint.*;
import healthwatcher.business.healthguide.*;
import lib.concurrency.ConcurrencyManager;


/**
 * TODO - describe this file
 * 
 * @author Thiago Tonelli Bartolomei <thiago.bartolomei@gmail.com>
 */
public deployed cclass HWManagedSynchronization {

	private ConcurrencyManager employeeManager = new ConcurrencyManager();
	private ConcurrencyManager diseaseManager = new ConcurrencyManager();
	private ConcurrencyManager symptomManager = new ConcurrencyManager();
	
	public pointcut synchronizationPoints(Object o) :
		(execution(* EmployeeRecord.insert(Employee)) ||
		 execution(* DiseaseRecord.insert(DiseaseType)) ||
		 execution(* SymptomRecord.insert(Symptom)) )&& args(o);
	
	before(Object o) : synchronizationPoints(o) {
		ConcurrencyManager manager = getManager(o);
		if (manager != null) {
			manager.beginExecution(getKey(o));
		}
	}
	
	after(Object o) : synchronizationPoints(o) {
		ConcurrencyManager manager = getManager(o);
		if (manager != null) {
			manager.endExecution(getKey(o));
		}
	}
	
	protected ConcurrencyManager getManager(Object o) {
		if (o instanceof Employee) {
			return employeeManager;
		}
		if (o instanceof DiseaseType) {
			return diseaseManager;
		}
		if (o instanceof Symptom) {
			return symptomManager;
		}
		return null;
	}
	
	protected Object getKey(Object o) {
		if (o instanceof Employee) {
			return ((Employee) o).getLogin();
		}
		if (o instanceof DiseaseType) {
			return "" + ((DiseaseType) o).getCode();
		}
		if (o instanceof Symptom) {
			return "" + ((Symptom) o).getCode();
		}
		return null;
	}
}
