/**
 * Copyright (c) 2006 Macacos.org. All Rights Reserved
 *
 * This library is free software; you can redistribute it and/or modify it under the terms of the 
 * GNU Lesser General Public License as published by the Free Software Foundation; either version 
 * 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See 
 * the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this library;
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, 
 * MA  02110-1301  USA
 * 
 * You can find the license also here: http://www.gnu.org/copyleft/lesser.html
 * 
 * 
 * Created on Sep 19, 2006 by Thiago Tonelli Bartolomei
 * -------------------------------------------------
 *           \                                                                           
 *            \                                                                          
 *               __                                                                      
 *          w  c(..)o                                                                    
 *           \__(o)                                                                      
 *               /\                                                                      
 *            w_/(_)-~                                                                   
 *                /|                                                                     
 *               | \                                                                     
 *               m  m   
 */
package healthwatcher.aspects.exceptionHandling;

import healthwatcher.view.servlets.*;
import javax.servlet.http.*;

import healthwatcher.data.rdb.AddressRepositoryRDB;
import healthwatcher.data.rdb.ComplaintRepositoryRDB;
import healthwatcher.model.address.Address;
import healthwatcher.model.complaint.Complaint;
import healthwatcher.view.servlets.HWServlet;

import java.io.IOException;
import java.io.PrintWriter;
//import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import lib.exceptions.ConnectionPersistenceMechanismException;
import lib.exceptions.ExceptionMessages;
import lib.exceptions.PersistenceMechanismException;
import lib.exceptions.RepositoryException;
import lib.exceptions.SQLPersistenceMechanismException;
import lib.exceptions.SoftSQLException;
import lib.persistence.PersistenceMechanism;
import lib.util.HTMLCode;

import org.aspectj.lang.SoftException;

/**
 * TODO - describe this file
 * 
 * @author Thiago Tonelli Bartolomei <thiago.bartolomei@gmail.com>
 */
public deployed cclass HWPersistenceExceptionHandler {
	
	protected Map persistenceLocalVariable = new HashMap();
	
	void around(HttpServletResponse response) : 
		execution(* HWServlet+.do*(HttpServletRequest, HttpServletResponse)) &&
		args(.., response) {
		
		try {
			
			proceed(response);
			
        } catch (RepositoryException e) {
        	try {
        		PrintWriter out = response.getWriter();
                out.println("</select></p></center></div>");
                out.println("<P> " + e.getMessage() + " </P>");
        	} catch (IOException ex) {
        		throw new RuntimeException(ex);
        	}
		}
	}
	
	//	----------------------------------AddressRepositoryRDB------------------------

	before(String sql): (call(* Statement.executeQuery(String))&& args(sql) ) && withincode(public Address AddressRepositoryRDB.search(int)) {
		persistenceLocalVariable.put(String.valueOf(Thread.currentThread().getId()), sql);
	}

	pointcut searchAddress() : execution(public Address AddressRepositoryRDB.search(int));
	
	after() throwing (SoftSQLException e) : searchAddress() {
		
		throw new SQLPersistenceMechanismException(ExceptionMessages.EXC_FALHA_BD,
				(String) persistenceLocalVariable.get(String
						.valueOf(Thread.currentThread().getId())));
	}

	after() throwing (PersistenceMechanismException e) /*throws RepositoryException*/ : searchAddress() {
		throw new RepositoryException(ExceptionMessages.EXC_FALHA_BD);
	}
	
	//----------------------------------ComplaintRepositoryRDB.update------------------------
	
	after() throwing (PersistenceMechanismException e) /*throws RepositoryException*/ : 
		execution(public void ComplaintRepositoryRDB.update(Complaint)) {
		
		throw new RepositoryException(ExceptionMessages.EXC_FALHA_ATUALIZACAO);
	}
	
	pointcut internalUpdateComplaint() : execution(* ComplaintRepositoryRDB.internal_update(Complaint, String));
	
	before(String sql): (call(* Statement.executeUpdate(String))&& args(sql) ) && 
		withincode(* ComplaintRepositoryRDB.internal_update(Complaint, String)) {
		
		persistenceLocalVariable.put(String.valueOf(Thread.currentThread().getId()), sql);
	}

	after() throwing (SoftSQLException e) : internalUpdateComplaint() {
		
		throw new SQLPersistenceMechanismException(ExceptionMessages.EXC_FALHA_BD,
				(String) persistenceLocalVariable.get(String
						.valueOf(Thread.currentThread().getId())));
	}
	
	//	----------------------------------ComplaintRepositoryRDB.insert------------------------
	
	pointcut insertComplaint() : execution(public int ComplaintRepositoryRDB.insert(Complaint));
	
	after() throwing (PersistenceMechanismException e) /*throws RepositoryException*/ : insertComplaint() {
		throw new RepositoryException(ExceptionMessages.EXC_FALHA_INCLUSAO);
	}

	before(String consulta): (call(* Statement.executeQuery(String)) && args(consulta) ) && 
		withincode(public int ComplaintRepositoryRDB.insert(Complaint)) {
		
		persistenceLocalVariable.put(String.valueOf(Thread.currentThread().getId()), consulta);
	}

	after() throwing (SoftSQLException e) : insertComplaint() {
		throw new SQLPersistenceMechanismException(ExceptionMessages.EXC_FALHA_BD,
				(String) persistenceLocalVariable.get(String
						.valueOf(Thread.currentThread().getId())));
	}
	

	// ----------------------------------PersistenceMechanism.connect---------------------------------
	
	pointcut connectPersistenceMechanism() : execution(* PersistenceMechanism.connect());
	
	void around(PersistenceMechanism obj) :	connectPersistenceMechanism() && this(obj) {
		
		try {
			proceed(obj);
		} catch (Exception e) {
			obj.incCurrentMirror();
			try {
				obj.connect();
			} catch (Exception ex1) {
				ex1.printStackTrace();
				throw new ConnectionPersistenceMechanismException(
						ExceptionMessages.EXC_FALHA_GET_CANAL_COMUNICACAO, obj.getUrl());
			}
		}
	}

	//----------------------------------PersistenceMechanism.getCommunicationChannel()---------------------------------
	
	pointcut getCommunicationChannel() :execution(Object PersistenceMechanism.getCommunicationChannel());
	
	Object around(PersistenceMechanism obj) : getCommunicationChannel() && this(obj) {
		
		try {
			return proceed(obj);
		} catch (Exception e) {
			obj.incCurrentMirror();
			try {
				obj.connect();
				return obj.getCommunicationChannel(false).createStatement();
			} catch (Exception ex1) {
				ex1.printStackTrace();
				throw new ConnectionPersistenceMechanismException(
						ExceptionMessages.EXC_FALHA_GET_CANAL_COMUNICACAO, obj.getUrl());
			}
		}
	}
}
